/**
 * SPDX-FileCopyrightText: Copyright 2023 Open Mobile Platform LLC <community@omp.ru>
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include <embedder_texture/camera.h>
#include <embedder_texture/camera_egl_helper.h>
#include <embedder_texture/camera_pixels_helper.h>

Camera::Camera(TextureRegistrar *plugin)
    : m_plugin(plugin)
    , m_manager(StreamCameraManager())
{
    CameraEGLHelper::EGLInit();
}

void Camera::InitializeCamera(int cameraID)
{
    if (m_cameraId != cameraID && m_manager->getNumberOfCameras()) {
        Aurora::StreamCamera::CameraInfo info;

        if (m_manager->getCameraInfo(cameraID, info)) {
            m_cameraId = cameraID;
            m_camera = m_manager->openCamera(info.id);
            m_camera->setListener(this);
        }
    }
}

Aurora::StreamCamera::CameraCapability Camera::StartCapture()
{
    if (m_camera) {
        Aurora::StreamCamera::CameraInfo info;

        if (m_camera->getInfo(info)) {
            std::vector<Aurora::StreamCamera::CameraCapability> caps;
            if (m_manager->queryCapabilities(info.id, caps)) {
                for(Aurora::StreamCamera::CameraCapability cap : caps) {
                    if (m_widthTexture + m_heightTexture < cap.width + cap.height) {
                        m_camera->startCapture(cap);
                        return cap;
                    }
                }
            }
        }
    }
    return Aurora::StreamCamera::CameraCapability{};
}

void Camera::StopCapture()
{
    if (m_camera) {
        m_cameraId = -1;
        m_buffer = nullptr;
        m_camera->stopCapture();
    }
}

std::map<Encodable, Encodable> Camera::Register(int cameraID, int width, int height)
{
    m_widthTexture = width;
    m_heightTexture = height;

    m_textureId = m_plugin->RegisterTexture(
        [this](size_t width, size_t height) { return this->m_buffer; });

    InitializeCamera(cameraID);
    auto cab = StartCapture();

    return std::map<Encodable, Encodable>{
        {"textureId", m_textureId},
        {"width", cab.width},
        {"height", cab.height},
    };
}

void Camera::Unregister()
{
    StopCapture();
    m_plugin->UnregisterTexture(m_textureId);
}

void Camera::onCameraFrame(std::shared_ptr<Aurora::StreamCamera::GraphicBuffer> buffer)
{
    if (buffer->handleType == Aurora::StreamCamera::HandleType::EGL) {
        // @todo Not tested. The device needs to be completed.
        auto eglImage = CameraEGLHelper::EGLCreateImage(buffer);
        this->m_buffer = new TextureVariant(FlutterEGLImage{
            eglImage,
            buffer->width,
            buffer->height,
        });
    } else {
        auto pixels = CameraPixelsHelper::YUVtoARGB(buffer->mapYCbCr());
        this->m_buffer = new TextureVariant(FlutterPixelBuffer{
            pixels,
            buffer->width,
            buffer->height,
        });
    }

    m_plugin->MarkTextureAvailable(m_textureId);
}

void Camera::onCameraError(const std::string &errorDescription)
{
    this->Unregister();
    std::cout << errorDescription << std::endl;
}

void Camera::onCameraParameterChanged(Aurora::StreamCamera::CameraParameter parameter,
                                      const std::string &value)
{
    std::cout << "onCameraParameterChanged" << std::endl;
}
